package nl.sbdeveloper.themeparkplus.util;

import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.BlockFace;
import org.bukkit.block.BlockState;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

public class LGUtil {
    private static boolean is113Plus = false;

    private static Method getBlockDataMethod;
    private static Method setBlockDataMethod;
    private static Method getAsStringMethod;
    private static Object lampAanData;
    private static Object lampUitData;

    static {
        try {
            Class<?> blockDataClass = Class.forName("org.bukkit.block.data.BlockData");
            Method methodCreateBlockData = Bukkit.class.getMethod("createBlockData", String.class);
            getBlockDataMethod = Block.class.getMethod("getBlockData");
            setBlockDataMethod = Block.class.getMethod("setBlockData", blockDataClass,
                    Boolean.TYPE);
            getAsStringMethod = blockDataClass.getMethod("getAsString");
            lampAanData = methodCreateBlockData.invoke(null, "minecraft:redstone_lamp[lit=true]");
            lampUitData = methodCreateBlockData.invoke(null, "minecraft:redstone_lamp[lit=false]");
            is113Plus = true;
        } catch (ClassNotFoundException | NoSuchMethodException | IllegalAccessException | InvocationTargetException ignored) {}
    }

    public static boolean zetLampAan(Block lampBlock) {
        if (is113Plus) {
            if ((isLamp(lampBlock)) && (!isAan(lampBlock))) {
                setBlockData(lampBlock, lampAanData);

                return true;
            }
            return false;
        }

        if (!isAan(lampBlock)) {
            final Block neighbor = getNeighbor(lampBlock);
            if (neighbor != null) {
                BlockState neighborState = neighbor.getState();
                try {
                    neighbor.setType(Material.REDSTONE_BLOCK);
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    neighborState.update(true, false);
                }
                return true;
            }
        }

        return false;
    }

    public static boolean zetLampUit(Block lampBlock) {
        if (is113Plus) {
            if ((isLamp(lampBlock)) && (isAan(lampBlock))) {
                setBlockData(lampBlock, lampUitData);

                return true;
            }
            return false;
        }

        if (isAan(lampBlock)) {
            lampBlock.setType(Objects.requireNonNull(Material.matchMaterial(XMaterial.REDSTONE_LAMP.getLegacy()[1])));
            return true;
        }
        return false;
    }

    public static boolean openGate(Block b) {
        return openGate(b, getDirection(b));
    }

    public static boolean openGate(Block b, BlockFace openFace) {
        if (is113Plus) {
            org.bukkit.block.data.BlockData blockData = b.getBlockData();
            if (isOpenable(b)) {
                org.bukkit.block.data.Openable op = (org.bukkit.block.data.Openable) blockData;
                if (op.isOpen()) return false;
                op.setOpen(true);
                b.setBlockData(blockData);

                org.bukkit.block.data.Directional dir = (org.bukkit.block.data.Directional) blockData;
                dir.setFacing(openFace);
                b.setBlockData(blockData);
                return true;
            }
        } else {
            BlockState state = b.getState();
            if (isOpenable(b)) {
                org.bukkit.material.Openable openable = (org.bukkit.material.Openable) state.getData();
                if (openable.isOpen()) return false;
                openable.setOpen(true);
                state.setData((org.bukkit.material.MaterialData) openable);
                state.update();

                org.bukkit.material.Directional dir = (org.bukkit.material.Directional) state.getData();
                dir.setFacingDirection(openFace);
                state.setData((org.bukkit.material.MaterialData) dir);
                state.update();
                return true;
            }
        }
        return false;
    }

    public static boolean closeGate(Block b) {
        if (is113Plus) {
            org.bukkit.block.data.BlockData blockData = b.getBlockData();
            if (isOpenable(b)) {
                org.bukkit.block.data.Openable op = (org.bukkit.block.data.Openable) blockData;
                if (!op.isOpen()) {
                    return false;
                }
                op.setOpen(false);
                b.setBlockData(blockData);
                return true;
            }
        } else {
            BlockState state = b.getState();
            if (isOpenable(b)) {
                org.bukkit.material.Openable openable = (org.bukkit.material.Openable) state.getData();
                if (!openable.isOpen()) {
                    return false;
                }
                openable.setOpen(false);
                state.setData((org.bukkit.material.MaterialData) openable);
                state.update();
                return true;
            }
        }
        return false;
    }

    public static BlockFace getDirection(Block b) {
        if (is113Plus) {
            org.bukkit.block.data.BlockData blockData = b.getBlockData();
            if (isOpenable(b)) {
                org.bukkit.block.data.Directional dir = (org.bukkit.block.data.Directional) blockData;
                return dir.getFacing();
            }
        } else {
            BlockState state = b.getState();
            if (isOpenable(b)) {
                org.bukkit.material.Directional dir = (org.bukkit.material.Directional) state.getData();
                return dir.getFacing();
            }
        }
        return null;
    }

    public static boolean isOpen(Block b) {
        if (is113Plus) {
            org.bukkit.block.data.BlockData blockData = b.getBlockData();
            if (isOpenable(b)) {
                org.bukkit.block.data.Openable op = (org.bukkit.block.data.Openable) blockData;
                return op.isOpen();
            }
        } else {
            BlockState state = b.getState();
            if (isOpenable(b)) {
                org.bukkit.material.Openable openable = (org.bukkit.material.Openable) state.getData();
                return openable.isOpen();
            }
        }
        return false;
    }

    /* Gate codes sponsored by MrWouter <3 */
    public static boolean isOpenable(Block b) {
        if (b == null) {
            return false;
        }
        if (is113Plus) {
            return b.getBlockData() instanceof org.bukkit.block.data.Openable;
        } else {
            return b.getState().getData() instanceof org.bukkit.material.Openable;
        }
    }

    private static Block getNeighbor(Block lampBlock) {
        List<Block> possibleNeighbors = new ArrayList<>();
        if (lampBlock.getLocation().getY() > 0.0D) {
            possibleNeighbors.add(lampBlock.getLocation().add(0.0D, -1.0D, 0.0D).getBlock());
        }
        if (lampBlock.getLocation().getY() < 255.0D) {
            possibleNeighbors.add(lampBlock.getLocation().add(0.0D, 1.0D, 0.0D).getBlock());
        }
        possibleNeighbors.add(lampBlock.getLocation().add(0.0D, 0.0D, 1.0D).getBlock());
        possibleNeighbors.add(lampBlock.getLocation().add(0.0D, 0.0D, -1.0D).getBlock());
        possibleNeighbors.add(lampBlock.getLocation().add(1.0D, 0.0D, 0.0D).getBlock());
        possibleNeighbors.add(lampBlock.getLocation().add(-1.0D, 0.0D, 0.0D).getBlock());

        for (Block neighbor: possibleNeighbors) {
            if (neighbor.getType() == Material.AIR) {
                return neighbor;
            }
        }

        for (Block neighbor: possibleNeighbors) {
            if ((neighbor.getState().getClass().getSimpleName().equals("CraftBlockState")) && (neighbor.getType() != Material.valueOf(XMaterial.PISTON_HEAD.getLegacy()[0]) && neighbor.getType() != Material.valueOf(XMaterial.MOVING_PISTON.getLegacy()[1])) && (neighbor.getType() != Material.valueOf("REDSTONE_LAMP_ON"))) {
                return neighbor;
            }
        }

        for (Block neighbor: possibleNeighbors) {
            if ((neighbor.getState().getClass().getSimpleName().equals("CraftBlockState")) && (neighbor.getType() != Material.valueOf(XMaterial.PISTON_HEAD.getLegacy()[0]) && neighbor.getType() != Material.valueOf(XMaterial.MOVING_PISTON.getLegacy()[1]))) {
                return neighbor;
            }
        }
        return null;
    }

    private static boolean isAan(Block lamp) {
        if (is113Plus) {
            return getAsString(getBlockData(lamp)).contains("lit=true");
        } else {
            return lamp.getType() == Material.matchMaterial(XMaterial.REDSTONE_LAMP.getLegacy()[0]);
        }
    }

    private static boolean isLamp(Block lamp) {
        if (is113Plus) {
            return lamp.getType() == XMaterial.REDSTONE_LAMP.parseMaterial();
        } else {
            return Arrays.asList(XMaterial.REDSTONE_LAMP.getLegacy()).contains(lamp.getType().name());
        }
    }

    private static void setBlockData(Block lamp, Object blockData) {
        try {
            setBlockDataMethod.invoke(lamp, blockData,
                    Boolean.FALSE);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static Object getBlockData(Block lamp) {
        try {
            return getBlockDataMethod.invoke(lamp);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private static String getAsString(Object blockData) {
        try {
            return (String) getAsStringMethod.invoke(blockData, new Object[0]);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

}